@@ -5,15 +5,16 @@ our ($ROUND, $PREC);
 
 Class::Multimethods::multimethod __ceil__ => qw(Math::MPFR) => sub {
     my ($x) = @_;
-    Math::MPFR::Rmpfr_ceil($x, $x);
-    $x;
+    my $r = Math::MPFR::Rmpfr_init2($PREC);
+    Math::MPFR::Rmpfr_ceil($r, $x);
+    $r;
 };
 
 Class::Multimethods::multimethod __ceil__ => qw(Math::GMPq) => sub {
     my ($x) = @_;
     my $z = Math::GMPz::Rmpz_init();
-    Math::GMPq::Rmpq_integer_p($x) && return $x;
     Math::GMPz::Rmpz_set_q($z, $x);
+    Math::GMPq::Rmpq_integer_p($x) && return $z;
     Math::GMPz::Rmpz_add_ui($z, $z, 1) if Math::GMPq::Rmpq_sgn($x) > 0;
     $z;
 };
@@ -34,8 +35,9 @@ Class::Multimethods::multimethod __ceil__ => qw(Math::MPC) => sub {
         return $real;
     }
 
-    Math::MPC::Rmpc_set_fr_fr($x, $real, $imag, $ROUND);
-    $x;
+    my $r = Math::MPC::Rmpc_init2($PREC);
+    Math::MPC::Rmpc_set_fr_fr($r, $real, $imag, $ROUND);
+    $r;
 };
 
 1;
