@@ -14,8 +14,7 @@ sub __eta__ {
 
     # Special case for eta(1) = log(2)
     if (Math::MPFR::Rmpfr_cmp_ui($x, 1) == 0) {
-        Math::MPFR::Rmpfr_add_ui($r, $x, 1, $ROUND);
-        Math::MPFR::Rmpfr_log($r, $r, $ROUND);
+        Math::MPFR::Rmpfr_const_log2($r, $ROUND);
         return $r;
     }
 
