@@ -7,16 +7,17 @@ Class::Multimethods::multimethod __imod__ => qw(Math::GMPz Math::GMPz) => sub {
     my $sign_y = Math::GMPz::Rmpz_sgn($y)
       || goto &Math::AnyNum::_nan;
 
-    Math::GMPz::Rmpz_mod($x, $x, $y);
+    my $r = Math::GMPz::Rmpz_init();
+    Math::GMPz::Rmpz_mod($r, $x, $y);
 
-    if (!Math::GMPz::Rmpz_sgn($x)) {
+    if (!Math::GMPz::Rmpz_sgn($r)) {
         ## OK
     }
     elsif ($sign_y < 0) {
-        Math::GMPz::Rmpz_add($x, $x, $y);
+        Math::GMPz::Rmpz_add($r, $r, $y);
     }
 
-    $x;
+    $r;
 };
 
 Class::Multimethods::multimethod __imod__ => qw(Math::GMPz $) => sub {
@@ -28,16 +29,17 @@ Class::Multimethods::multimethod __imod__ => qw(Math::GMPz $) => sub {
     my $neg_y = $y < 0;
     $y = -$y if $neg_y;
 
-    Math::GMPz::Rmpz_mod_ui($x, $x, $y);
+    my $r = Math::GMPz::Rmpz_init();
+    Math::GMPz::Rmpz_mod_ui($r, $x, $y);
 
-    if (!Math::GMPz::Rmpz_sgn($x)) {
+    if (!Math::GMPz::Rmpz_sgn($r)) {
         ## OK
     }
     elsif ($neg_y) {
-        Math::GMPz::Rmpz_sub_ui($x, $x, $y);
+        Math::GMPz::Rmpz_sub_ui($r, $r, $y);
     }
 
-    $x;
+    $r;
 };
 
 1;
