@@ -7,15 +7,16 @@ use warnings;
 sub __ipow__ {
     my ($x, $y) = @_;
 
-    Math::GMPz::Rmpz_pow_ui($x, $x, CORE::abs($y));
+    my $r = Math::GMPz::Rmpz_init();
+    Math::GMPz::Rmpz_pow_ui($r, $x, CORE::abs($y));
 
     if ($y < 0) {
-        Math::GMPz::Rmpz_sgn($x) || goto &Math::AnyNum::_inf;    # 0^(-y) = Inf
+        Math::GMPz::Rmpz_sgn($r) || goto &Math::AnyNum::_inf;    # 0^(-y) = Inf
         state $ONE_Z = Math::GMPz::Rmpz_init_set_ui_nobless(1);
-        Math::GMPz::Rmpz_tdiv_q($x, $ONE_Z, $x);
+        Math::GMPz::Rmpz_tdiv_q($r, $ONE_Z, $r);
     }
 
-    $x;
+    $r;
 }
 
 1;
