@@ -12,29 +12,32 @@ sub __iroot__ {
 
         # 1^Inf = 1 ; (-1)^Inf = 1
         if (Math::GMPz::Rmpz_cmpabs_ui($x, 1) == 0) {
-            Math::GMPz::Rmpz_abs($x, $x);
-            return $x;
+            return Math::GMPz::Rmpz_init_set_ui(1);
         }
 
         goto &Math::AnyNum::_inf;
     }
     elsif ($y < 0) {
-        my $sign = Math::GMPz::Rmpz_sgn($x) || goto &Math::AnyNum::_inf;    # 1 / 0^k = Inf
-        Math::GMPz::Rmpz_cmp_ui($x, 1) == 0 and return $x;                  # 1 / 1^k = 1
+        my $sign = Math::GMPz::Rmpz_sgn($x)
+          || goto &Math::AnyNum::_inf;            # 1 / 0^k = Inf
 
         if ($sign < 0) {
             goto &Math::AnyNum::_nan;
         }
 
-        Math::GMPz::Rmpz_set_ui($x, 0);
-        return $x;
+        if (Math::GMPz::Rmpz_cmp_ui($x, 1) == 0) {    # 1 / 1^k = 1
+            return $x;
+        }
+
+        return Math::GMPz::Rmpz_init_set_ui(0);
     }
     elsif ($y % 2 == 0 and Math::GMPz::Rmpz_sgn($x) < 0) {
         goto &Math::AnyNum::_nan;
     }
 
-    Math::GMPz::Rmpz_root($x, $x, $y);
-    $x;
+    my $r = Math::GMPz::Rmpz_init();
+    Math::GMPz::Rmpz_root($r, $x, $y);
+    $r;
 }
 
 1;
