@@ -16,8 +16,9 @@ Class::Multimethods::multimethod __log__ => qw(Math::MPFR) => sub {
         goto &__log__;
     }
 
-    Math::MPFR::Rmpfr_log($x, $x, $ROUND);
-    $x;
+    my $r = Math::MPFR::Rmpfr_init2($PREC);
+    Math::MPFR::Rmpfr_log($r, $x, $ROUND);
+    $r;
 };
 
 Class::Multimethods::multimethod __log2__ => qw(Math::MPFR) => sub {
@@ -29,8 +30,9 @@ Class::Multimethods::multimethod __log2__ => qw(Math::MPFR) => sub {
         goto &__log2__;
     }
 
-    Math::MPFR::Rmpfr_log2($x, $x, $ROUND);
-    $x;
+    my $r = Math::MPFR::Rmpfr_init2($PREC);
+    Math::MPFR::Rmpfr_log2($r, $x, $ROUND);
+    $r;
 };
 
 Class::Multimethods::multimethod __log10__ => qw(Math::MPFR) => sub {
@@ -42,8 +44,9 @@ Class::Multimethods::multimethod __log10__ => qw(Math::MPFR) => sub {
         goto &__log10__;
     }
 
-    Math::MPFR::Rmpfr_log10($x, $x, $ROUND);
-    $x;
+    my $r = Math::MPFR::Rmpfr_init2($PREC);
+    Math::MPFR::Rmpfr_log10($r, $x, $ROUND);
+    $r;
 };
 
 #
@@ -51,18 +54,19 @@ Class::Multimethods::multimethod __log10__ => qw(Math::MPFR) => sub {
 #
 
 Class::Multimethods::multimethod __log__ => qw(Math::MPC) => sub {
-    my ($x) = @_;
-    Math::MPC::Rmpc_log($x, $x, $ROUND);
-    $x;
+    my $r = Math::MPC::Rmpc_init2($PREC);
+    Math::MPC::Rmpc_log($r, $_[0], $ROUND);
+    $r;
 };
 
 Class::Multimethods::multimethod __log2__ => qw(Math::MPC) => sub {
     my ($x) = @_;
+    my $r   = Math::MPC::Rmpc_init2($PREC);
     my $ln2 = Math::MPFR::Rmpfr_init2($PREC);
     Math::MPFR::Rmpfr_const_log2($ln2, $ROUND);
-    Math::MPC::Rmpc_log($x, $x, $ROUND);
-    Math::MPC::Rmpc_div_fr($x, $x, $ln2, $ROUND);
-    $x;
+    Math::MPC::Rmpc_log($r, $x, $ROUND);
+    Math::MPC::Rmpc_div_fr($r, $r, $ln2, $ROUND);
+    $r;
 };
 
 Class::Multimethods::multimethod __log10__ => qw(Math::MPC) => sub {
@@ -70,18 +74,20 @@ Class::Multimethods::multimethod __log10__ => qw(Math::MPC) => sub {
 
     state $MPC_VERSION = Math::MPC::MPC_VERSION();
 
+    my $r = Math::MPC::Rmpc_init2($PREC);
+
     if ($MPC_VERSION >= 65536) {    # available only in mpc>=1.0.0
-        Math::MPC::Rmpc_log10($x, $x, $ROUND);
+        Math::MPC::Rmpc_log10($r, $x, $ROUND);
     }
     else {
         my $ln10 = Math::MPFR::Rmpfr_init2($PREC);
         Math::MPFR::Rmpfr_set_ui($ln10, 10, $ROUND);
         Math::MPFR::Rmpfr_log($ln10, $ln10, $ROUND);
-        Math::MPC::Rmpc_log($x, $x, $ROUND);
-        Math::MPC::Rmpc_div_fr($x, $x, $ln10, $ROUND);
+        Math::MPC::Rmpc_log($r, $x, $ROUND);
+        Math::MPC::Rmpc_div_fr($r, $r, $ln10, $ROUND);
     }
 
-    $x;
+    $r;
 };
 
 1;
