@@ -8,18 +8,19 @@ Class::Multimethods::multimethod __sqrt__ => qw(Math::MPFR) => sub {
 
     # Complex for x < 0
     if (Math::MPFR::Rmpfr_sgn($x) < 0) {
-        (@_) = _mpfr2mpc($_[0]);
+        (@_) = _mpfr2mpc($x);
         goto &__sqrt__;
     }
 
-    Math::MPFR::Rmpfr_sqrt($x, $x, $ROUND);
-    $x;
+    my $r = Math::MPFR::Rmpfr_init2($PREC);
+    Math::MPFR::Rmpfr_sqrt($r, $x, $ROUND);
+    $r;
 };
 
 Class::Multimethods::multimethod __sqrt__ => qw(Math::MPC) => sub {
-    my ($x) = @_;
-    Math::MPC::Rmpc_sqrt($x, $x, $ROUND);
-    $x;
+    my $r = Math::MPC::Rmpc_init2($PREC);
+    Math::MPC::Rmpc_sqrt($r, $_[0], $ROUND);
+    $r;
 };
 
 1;
